----------------------------------------------------
-- PartyTargets
-- Shows who your party members are targeting.
----------------------------------------------------
local addon = LibStub:NewLibrary("PartyTargets-3.3", 200912123.3)
assert(addon, "Failed to load library 'PartyTargets-3.3' with LibStub")

-- OnLoad Function
addon.OnLoad = function(self)
	self:SetAttribute("unit", "party" .. self:GetID() .. "target")
	self:SetAttribute("type", "party" .. self:GetID() .. "target")
	RegisterUnitWatch(self)
	
	self.statusCounter = 0
	self.statusSign = -1
	self.unitHPPercent = 1
	
	addon.UpdateMember(self)
	
	-- Register Events
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("PARTY_MEMBER_ENABLE")
	self:RegisterEvent("PARTY_MEMBER_DISABLE")
	self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
	self:RegisterEvent("VARIABLES_LOADED")
	self:RegisterEvent("UNIT_FACTION")
	self:RegisterEvent("UNIT_TARGET")
	self:RegisterEvent("UNIT_PVP_UPDATE")
	self:RegisterEvent("UNIT_HEALTH")
	self:RegisterEvent("UNIT_MAXHEALTH")
	self:RegisterEvent("UNIT_MANA")
	self:RegisterEvent("UNIT_ENERGY")
	self:RegisterEvent("UNIT_FOCUS")
	self:RegisterEvent("UNIT_RAGE")
	self:RegisterEvent("UNIT_RUNIC_POWER")
	
	-- Initialize Dropdown Box
	local showmenu = function()
		ToggleDropDownMenu(1, nil, _G["PartyTargetFrame" .. self:GetID() .. "DropDown"], self:GetName(), 47, 15)
	end
	SecureUnitButton_OnLoad(self, "party" .. self:GetID() .. "target", showmenu)
end

-- OnEvent function
addon.OnEvent = function(self, e, ...)
	UnitFrame_OnEvent(self, e, ...)
	
	-- Get unit from data
	local unit = select(1, ...)
	if (e == "PLAYER_ENTERING_WORLD") then
		if (GetPartyMember(self:GetID())) then
			addon.UpdateMember(self)
			return
		end
	end
	
	if (e == "PARTY_MEMBERS_CHANGED" or e == "UNIT_TARGET") then
		addon.UpdateMember(self)
		return
	end
	
	if (unit) then
		for i = 1, MAX_PARTY_MEMBERS, 1 do
			if (unit == "party" .. i .. "target" and UnitExists("party"..i.."target")) then
				addon.UpdateMember(self)
			end
		end
	else
		addon.UpdateMember(self)
	end
end

-- Update Member function
addon.UpdateMember = function(self)
	if (GetPartyMember(self:GetID()) and UnitExists("party"..self:GetID().."target")) then
		UnitFrameManaBar_UpdateType(self.manabar)
		UnitFrame_Update(self)
	end
end

-- Update Member Health function
addon.UpdateMemberHealth = function(self, e)
	if ((self.unitHPPercent > 0) and (self.unitHPPercent <= 0.2)) then
		local alpha = 255
		local counter = self.statusCounter + e
		local sign = self.statusSign
		
		if (counter > 0.5) then
			sign = -sign
			self.statusSign = sign
		end
		
		counter = mod(counter, 0.5)
		self.statusCounter = counter
		
		if (sign == 1) then
			alpha = (127 + (counter * 256)) / 255
		else
			alpha = (255 - (counter * 256)) / 255
		end
		
		_G[self:GetName().."Portrait"]:SetAlpha(alpha)
	end
end

-- OnUpdate function
addon.OnUpdate = function(self, e)
	addon.UpdateMemberHealth(self, e)
end

-- Health Check function
addon.HealthCheck = function(self, value)
	local UnitHealth, UnitHealthMax = self:GetMinMaxValues()
	local UnitHealthCurrent = self:GetValue()
	
	if (UnitHealthMax > 0) then
		self:GetParent().unitHPPercent = UnitHealthCurrent / UnitHealthMax
	else
		self:GetParent().unitHPPercent = 0
	end
	
	_G[self:GetParent():GetName() .. "Portrait"]:SetVertexColor(1, 1, 1, 1)
end

-- Dropdown OnLoad function
addon.DropDownOnLoad = function(self)
	UIDropDownMenu_Initialize(self, addon.DropDownInitialize, "MENU")
end

-- Dropdown Initialize function
addon.DropDownInitialize = function(self)
	local dropdown = UIDROPDOWNMENU_OPEN_MENU or self
	UnitPopup_ShowMenu(dropdown, "TARGET", "party" .. dropdown:GetParent():GetID() .. "target")
end